/*
    AutoX11, a tool to automate tasks in an X11 environment
    Copyright (C) 2014  Alicia <alicia@ion.nu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <wchar.h>
#include <X11/extensions/XTest.h>
#include <X11/keysym.h>
#include <X11/XKBlib.h>
#include "autox11.h"

#define MOD_SHIFT 1
#define MOD_ALTGR 2
#define MOD_CTRL  4

void typechar(Display* display, wchar_t chr)
{
  // Handle character codes that don't match up against X11 keysyms
  switch(chr)
  {
    case L'\b': chr=XK_BackSpace; break;
    case L'\t': chr=XK_Tab; break;
    case L'\n': chr=XK_Return; break;
    case L'\r': chr=XK_Return; break;
    case L'\e': chr=XK_Escape; break;
    case 127: chr=XK_BackSpace; break;
  }
  // Figure out which modifiers are needed, if any
  char mods=0;
  KeyCode keycode=XKeysymToKeycode(display, (KeySym)chr);
  if(XkbKeycodeToKeysym(display, keycode, 0, 0)!=chr)
  {
    if(XkbKeycodeToKeysym(display, keycode, 0, MOD_SHIFT)==chr){mods=MOD_SHIFT;}
    else if(XkbKeycodeToKeysym(display, keycode, 0, MOD_ALTGR)==chr){mods=MOD_ALTGR;}
    else if(XkbKeycodeToKeysym(display, keycode, 0, MOD_SHIFT|MOD_ALTGR)==chr){mods=MOD_SHIFT|MOD_ALTGR;}
    else if(chr>0&&chr<=L'Z'-L'@'){chr+=L'@'; mods=MOD_CTRL;}
  }
  // Modifiers
  if(mods & MOD_SHIFT)
  {
    keycode=XKeysymToKeycode(display, XK_Shift_L);
    XTestFakeKeyEvent(display, keycode, 1, 0);
  }
  if(mods & MOD_ALTGR)
  {
    keycode=XKeysymToKeycode(display, XK_ISO_Level3_Shift);
    XTestFakeKeyEvent(display, keycode, 1, 0);
  }
  if(mods & MOD_CTRL)
  {
    keycode=XKeysymToKeycode(display, XK_Control_L);
    XTestFakeKeyEvent(display, keycode, 1, 0);
  }
  // Main key
  keycode=XKeysymToKeycode(display, (KeySym)chr);
  XTestFakeKeyEvent(display, keycode, 1, 0);
  XTestFakeKeyEvent(display, keycode, 0, 0);
  // Release modifiers
  if(mods & MOD_SHIFT)
  {
    keycode=XKeysymToKeycode(display, XK_Shift_L);
    XTestFakeKeyEvent(display, keycode, 0, 0);
  }
  if(mods & MOD_ALTGR)
  {
    keycode=XKeysymToKeycode(display, XK_ISO_Level3_Shift);
    XTestFakeKeyEvent(display, keycode, 0, 0);
  }
  if(mods & MOD_CTRL)
  {
    keycode=XKeysymToKeycode(display, XK_Control_L);
    XTestFakeKeyEvent(display, keycode, 0, 0);
  }
}
